"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.denormalizeDocument = denormalizeDocument;
var _HTypes = require("./HTypes");
var _HVisit = require("./HVisit");
var _HUtils = require("./HUtils");
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
const elementUid = (elementType, elementId) => `${elementType}:${elementId}`;
function denormalizeDocument(doc) {
  const nodes = new Map();
  // I will need two visits. One to create all the nodes while setting the parents, and
  // a second one where we change the children.
  (0, _HVisit.visitDocument)(doc, (normalizedDoc, nodeType, nodeId) => {
    const element = (0, _HUtils.mappedElement)(doc.maps, nodeType, nodeId);
    const parentUid = element.parentType && element.parentId ? elementUid(element.parentType, element.parentId) : null;
    const parent = parentUid ? nodes.get(parentUid) || null : null;
    const denormalizedNode = _objectSpread(_objectSpread({}, element), {}, {
      parent
    });
    nodes.set(elementUid(nodeType, nodeId), denormalizedNode);
  });
  (0, _HVisit.visitDocument)(doc, (normDoc, nodeType, nodeId) => {
    const element = (0, _HUtils.mappedElement)(doc.maps, nodeType, nodeId);
    const uid = elementUid(nodeType, nodeId);
    const denormalizedNode = nodes.get(uid);
    const typeLinks = doc.schema.types[nodeType];
    for (const fieldName in typeLinks) {
      if (fieldName === 'parentId') {
        continue;
      }
      const linkProps = typeLinks[fieldName];
      if (Array.isArray(linkProps)) {
        const linkedIds = element[fieldName];
        if (!Array.isArray(linkedIds)) {
          denormalizedNode[fieldName] = [];
        } else {
          denormalizedNode[fieldName] = linkedIds.map(childId => nodes.get(elementUid(linkProps[0].__schemaType, childId))).filter(childNode => childNode !== undefined);
        }
      } else {
        denormalizedNode[fieldName] = null;
      }
    }
  }, {
    context: {},
    traversal: _HTypes.DocumentVisitTraversal.DEPTH_FIRST
  });
  return nodes.get(elementUid(doc.rootType, doc.rootId));
}
//# sourceMappingURL=HDenormalize.js.map