import { AllMappedTypesFields, HDocOperation, Id, IElementId, IMutableDocument, INormalizedDocument, INormalizedMutableMapsDocument, IParentedId, MapsOfNormDoc, Path, UOfNormDoc } from './HTypes';
/**
 * Creates a shallow clone of the document. The maps are new objects,
 * but the entities mapped are the same as the original.
 *
 * The rationale for the shallow version of the elements is that
 * changes will be performed as setting new versions in the dictionary, rather
 * than direct manipulation of the objects.
 *
 * @param {INormalizedDocument<MapsInterface, U>} doc The document to be cloned
 * @returns {INormalizedDocument<MapsInterface, U>} shallow clone of the document
 */
export declare function cloneNormalizedDocument<NorDoc extends INormalizedDocument<any, any>>(doc: NorDoc): NorDoc;
/**
 * Creates an empty copy version of the normalized document
 * passed as parameter.
 *
 * @returns {INormalizedDocument<MapsInterface, U>}
 */
export declare function clearedNormalizedDocument<NorDoc extends INormalizedDocument<any, any>>(doc: NorDoc): NorDoc;
export declare function idAndTypeForPath<MapsInterface, U extends keyof MapsInterface>(doc: INormalizedDocument<MapsInterface, U> | INormalizedMutableMapsDocument<MapsInterface, U> | IMutableDocument<MapsInterface, U>, path: Path<MapsInterface>): {
    __typename: U;
    _id: Id;
};
export declare function parentToChildFieldName<MapsInterface, U extends keyof MapsInterface>(document: INormalizedDocument<MapsInterface, U> | INormalizedMutableMapsDocument<MapsInterface, U>, parentType: U, childType: U): AllMappedTypesFields<MapsInterface>;
/**
 * Given an element type and id of and element within the document,
 * returns the path to it.
 *
 * Each element is expected to be referenced only once as an official child
 * within the document.
 *
 * The element could still be referenced informally from fields not captured
 * in the document schema
 *
 * @param {U} elementTypeMap
 * @param {Id} elementId
 * @returns {Path}
 */
export declare function pathForElementWithId<NorDoc extends INormalizedDocument<any, any>>(doc: INormalizedDocument<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>> | INormalizedMutableMapsDocument<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>>, elementTypeMap: UOfNormDoc<NorDoc>, elementId: Id): Path<MapsOfNormDoc<NorDoc>>;
export declare function mutableDocument<NorDoc extends INormalizedDocument<any, any>>(doc: NorDoc): IMutableDocument<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>, NorDoc>;
export declare const docReducer: <NorDoc extends INormalizedDocument<any, any>>(doc: NorDoc, cmd: HDocOperation<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>, any> | HDocOperation<MapsOfNormDoc<NorDoc>, UOfNormDoc<NorDoc>, any>[]) => NorDoc;
export declare const removeElementFromArrayReducer: <NorDoc extends INormalizedDocument<any, any>, T>(doc: NorDoc, elementType: UOfNormDoc<NorDoc>, elementId: Id, arrayFieldName: AllMappedTypesFields<MapsOfNormDoc<NorDoc>>, arrayElement: T) => NorDoc;
/**
 * Creates a new document with an array field updated with the
 * additional element(s) passed as parameters, optionally from a specific
 * index.
 *
 * @param {NorDoc} doc
 * @param {UOfNormDoc<NorDoc>} elementType
 * @param {Id} elementId
 * @param {AllMappedTypesFields<MapsOfNormDoc<NorDoc>>} arrayFieldName
 * @param {T} arrayElement
 * @param {number} insertIntoIndex
 * @returns {NorDoc}
 */
export declare const addElementToArrayReducer: <NorDoc extends INormalizedDocument<any, any>, T>(doc: NorDoc, elementType: UOfNormDoc<NorDoc>, elementId: Id, arrayFieldName: AllMappedTypesFields<MapsOfNormDoc<NorDoc>>, arrayElement: T | T[], insertIntoIndex?: number) => NorDoc;
export declare function idAndTypeOfChange<MapsInterface, U extends keyof MapsInterface>(change: HDocOperation<MapsInterface, U, IParentedId<U, U>>, doc?: INormalizedDocument<MapsInterface, U> | IMutableDocument<MapsInterface, U>): IElementId<U>;
