import { ElementInfoConflicts, EqualFn, IDocumentSchema, II3MergeResult, IMergeOptions, INormalizedDocument, IParentedId, WasTouchedFn } from './HTypes';
interface IMergedElementInfoResult<MapsInterface, U extends keyof MapsInterface, ElementType extends IParentedId<U, U>> {
    mergedElement: ElementType;
    conflicts?: ElementInfoConflicts<ElementType>;
}
/**
 * Given a base version of a normalized document, and
 * two later versions of the document, generates a merge
 * tree reporting back if there were conflicts and a diff
 * object to go from myDoc to theirDoc.
 *
 * @param {INormalizedDocument<MapsInterface, U>} baseDoc
 * @param {INormalizedDocument<MapsInterface, U>} myDoc
 * @param {INormalizedDocument<MapsInterface, U>} theirDoc
 * @param {MergeOverridesMap<MapsInterface, U, ElementType>} options
 * @returns {II3MergeResult<MapsInterface, U>}
 */
export declare function threeWayMerge<NorDoc extends INormalizedDocument<any, any>>(baseDoc: NorDoc, myDoc: NorDoc, theirDoc: NorDoc, options?: IMergeOptions<NorDoc>): II3MergeResult<NorDoc>;
export declare function mergeElementInfo<MapsInterface, U extends keyof MapsInterface, ElementType extends IParentedId<U, U>, K extends keyof ElementType = keyof ElementType>(schema: IDocumentSchema<MapsInterface, U>, elementType: U, baseElement: ElementType, leftElement: ElementType, rightElement: ElementType, excludeFields?: K[]): IMergedElementInfoResult<MapsInterface, U, ElementType>;
export declare function threeWayMergeArray<T>(base: T[], mine: T[], their: T[], props?: {
    equalsFn?: EqualFn;
    wasTouchedFn?: WasTouchedFn<T>;
}): T[];
export {};
