import { EntitiesMaps, Id, IElementId, IFieldEntityReference, ILazyMutableMap, IMutableDocument, INormalizedDocument, IParentedId, MutableEntitiesMaps, SubEntityPathElement } from './HTypes';
export declare function isId(obj: any): obj is Id;
export declare function isElementId(obj: any): obj is IElementId<any>;
export declare function isNullableId(obj: any): obj is Id | null;
export declare function isParentedId(obj: any): obj is IParentedId;
/**
 * Check if a map contains parented id elements. For performance reasons
 * we only check the object is a map.
 * @param obj
 * @returns {obj is Map<Id, IParentedId>}
 */
export declare function isParentedMap(obj: any): obj is Map<Id, IParentedId>;
export declare function isEntitiesMap(obj: any): obj is EntitiesMaps<any, any>;
export declare function isSubEntityPathElement(obj: any): obj is SubEntityPathElement<any>;
export declare function isSchemaReference(obj: any): obj is IFieldEntityReference<any>;
export declare function isIParentedId<U>(obj: any): obj is IParentedId<U>;
/**
 * Checks if the object passed as parameter is a mutable
 * lazy map with Ids as keys and IParentedId subtypes as
 * values.
 *
 * @param obj
 * @returns {obj is ILazyMutableMap<Id, IParentedId>}
 */
export declare function isParentedMutableMap(obj: any): obj is ILazyMutableMap<Id, IParentedId>;
export declare function isMutableEntitiesMap(obj: any): obj is MutableEntitiesMaps<any, any>;
/**
 * Checks if an element of type elementType with id elementId is in the document
 * hierarchy.
 *
 * @param {EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U>} docOrMaps
 * @param {U} elementType
 * @param {Id} elementId
 * @returns {boolean}
 */
export declare function hasMappedElement<MapsInterface, U extends keyof EntitiesMaps<MapsInterface> = keyof EntitiesMaps<MapsInterface>>(docOrMaps: EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U> | INormalizedDocument<MapsInterface, U> | IMutableDocument<MapsInterface, U>, elementType: U, elementId: Id): boolean;
/**
 * Given a map of entities or mutable entities, returns the element that corresponds to the type
 * and id provided as parameter.
 *
 * If the type requested is not mapped, a type error is thrown. If no element with the requested _id is found,
 * a reference error is thrown.
 *
 * @param {EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U>} maps
 * @param {U} elementType
 * @param {Id} elementId
 * @returns {EntitiesMaps<MapsInterface>[U] extends Map<Id, infer T> ? T : never}
 */
export declare function mappedElement<MapsInterface, U extends keyof EntitiesMaps<MapsInterface> = keyof EntitiesMaps<MapsInterface>>(docOrMaps: EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U> | INormalizedDocument<MapsInterface, U> | IMutableDocument<MapsInterface, U>, elementType: U, elementId: Id): EntitiesMaps<MapsInterface>[U] extends Map<Id, infer T> ? T : never;
/**
 * Generates a unique id to assign to an element.
 *
 * @returns {Id}
 */
export declare const generateNewId: () => Id;
export interface AssertConf {
    outputFn: (text: string) => void;
    throwOnViolation: boolean;
}
export declare function setAssertOptions(assertSettings: Partial<AssertConf>): void;
export declare function assert(predicate: boolean, errorMsg: string): void;
