"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.assert = assert;
exports.generateNewId = void 0;
exports.hasMappedElement = hasMappedElement;
exports.isElementId = isElementId;
exports.isEntitiesMap = isEntitiesMap;
exports.isIParentedId = isIParentedId;
exports.isId = isId;
exports.isMutableEntitiesMap = isMutableEntitiesMap;
exports.isNullableId = isNullableId;
exports.isParentedId = isParentedId;
exports.isParentedMap = isParentedMap;
exports.isParentedMutableMap = isParentedMutableMap;
exports.isSchemaReference = isSchemaReference;
exports.isSubEntityPathElement = isSubEntityPathElement;
exports.mappedElement = mappedElement;
exports.setAssertOptions = setAssertOptions;
var _uuid = require("uuid");
var _LazyMap = require("./LazyMap");
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return typeof key === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (typeof input !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (typeof res !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
function isId(obj) {
  return typeof obj === 'number' || obj !== '' && typeof obj === 'string';
}
function isElementId(obj) {
  return typeof obj === 'object' && isId(obj._id) && typeof obj.__typename === 'string';
}
function isNullableId(obj) {
  return obj === null || isId(obj);
}
function isParentedId(obj) {
  return typeof obj === 'object' && isId(obj._id) && typeof obj.__typename === 'string' && (obj.parentType === null || typeof obj.parentType === 'string') && (obj.parentId === null || isId(obj.parentId));
}

/**
 * Check if a map contains parented id elements. For performance reasons
 * we only check the object is a map.
 * @param obj
 * @returns {obj is Map<Id, IParentedId>}
 */
function isParentedMap(obj) {
  return Boolean(obj && obj instanceof Map);
}
function isEntitiesMap(obj) {
  if (typeof obj !== 'object') return false;
  let i = 0;
  for (const entityType of Object.keys(obj)) {
    if (!isParentedMap(obj[entityType])) {
      return false;
    }
    i++;
  }
  return i > 0;
}
function isSubEntityPathElement(obj) {
  if (typeof obj === 'string') {
    return true;
  }
  if (Array.isArray(obj) && obj.length === 2 && typeof obj[0] === 'string' && typeof obj[1] === 'number') {
    return true;
  }
  return false;
}
function isSchemaReference(obj) {
  return Boolean(obj && typeof obj === 'object' && '__schemaType' in obj && obj.__schemaType && typeof obj.__schemaType === 'string');
}
function isIParentedId(obj) {
  return obj && typeof obj === 'object' && obj._id !== undefined && isId(obj._id) && (typeof obj.parentId === 'undefined' || isId(obj.parentId) || obj.parentId === null);
}

/**
 * Checks if the object passed as parameter is a mutable
 * lazy map with Ids as keys and IParentedId subtypes as
 * values.
 *
 * @param obj
 * @returns {obj is ILazyMutableMap<Id, IParentedId>}
 */
function isParentedMutableMap(obj) {
  return Boolean(obj && obj instanceof _LazyMap.LazyMutableMap);
}
function isMutableEntitiesMap(obj) {
  if (typeof obj !== 'object') return false;
  const fieldNames = Object.keys(obj);
  if (fieldNames.length < 1) return false;
  for (const fieldName of fieldNames) {
    if (!isParentedMutableMap(obj[fieldName])) {
      return false;
    }
  }
  return true;
}

/**
 * Checks if an element of type elementType with id elementId is in the document
 * hierarchy.
 *
 * @param {EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U>} docOrMaps
 * @param {U} elementType
 * @param {Id} elementId
 * @returns {boolean}
 */
function hasMappedElement(docOrMaps, elementType, elementId) {
  let maps;
  if (isEntitiesMap(docOrMaps) || isMutableEntitiesMap(docOrMaps)) {
    maps = docOrMaps;
  } else if (isEntitiesMap(docOrMaps.maps) || isMutableEntitiesMap(docOrMaps.maps)) {
    maps = docOrMaps.maps;
  } else {
    return false;
  }
  if (!(elementType in maps)) {
    return false;
  }
  const typeMap = maps[elementType];
  return typeMap.has(elementId);
}

/**
 * Given a map of entities or mutable entities, returns the element that corresponds to the type
 * and id provided as parameter.
 *
 * If the type requested is not mapped, a type error is thrown. If no element with the requested _id is found,
 * a reference error is thrown.
 *
 * @param {EntitiesMaps<MapsInterface, U> | MutableEntitiesMaps<MapsInterface, U>} maps
 * @param {U} elementType
 * @param {Id} elementId
 * @returns {EntitiesMaps<MapsInterface>[U] extends Map<Id, infer T> ? T : never}
 */
function mappedElement(docOrMaps, elementType, elementId) {
  let maps;
  if (isEntitiesMap(docOrMaps) || isMutableEntitiesMap(docOrMaps)) {
    maps = docOrMaps;
  } else if (isEntitiesMap(docOrMaps.maps) || isMutableEntitiesMap(docOrMaps.maps)) {
    maps = docOrMaps.maps;
  } else {
    maps = {};
  }
  if (!(elementType in maps)) {
    throw new TypeError(`Element type ${String(elementType)} not found`);
  }
  const typeMap = maps[elementType];
  if (!typeMap.has(elementId)) {
    throw new ReferenceError(`Referential integrity: element ${String(elementType)}.${elementId} not found`);
  }
  return typeMap.get(elementId);
}

/**
 * Generates a unique id to assign to an element.
 *
 * @returns {Id}
 */
const generateNewId = () => (0, _uuid.v1)();
exports.generateNewId = generateNewId;
let assertConf = {
  outputFn: console.log,
  throwOnViolation: true
};
function setAssertOptions(assertSettings) {
  assertConf = _objectSpread(_objectSpread({}, assertConf), assertSettings);
}
function assert(predicate, errorMsg) {
  if (!predicate) {
    if (assertConf.outputFn) {
      assertConf.outputFn(errorMsg);
    }
    if (assertConf.throwOnViolation) {
      throw new Error(errorMsg);
    }
  }
}
//# sourceMappingURL=HUtils.js.map